#include "QtCameraWidget.h"

Q_DECLARE_METATYPE(QCameraInfo)

QImage img_no_scaled;

QtCameraWidget::QtCameraWidget(QWidget *parent)
	: QWidget(parent)
{
	ui.setupUi(this);

	imagecapture = new QCameraImageCapture(m_camera.data());

	const QList<QCameraInfo> cameras = QCameraInfo::availableCameras();

	for (const QCameraInfo& cameraInfo : cameras)
		ui.CameraComboBox->addItem(cameraInfo.description(), QVariant::fromValue(cameraInfo));

	connect(ui.CameraComboBox, SIGNAL(currentIndexChanged(int)), SLOT(CameraChange(int)));
	connect(ui.imageResolutionBox, SIGNAL(currentIndexChanged(int)), SLOT(DoResChange(int)));

	if (ui.CameraComboBox->count() > 0)
		SetCamera(QCameraInfo::defaultCamera());

	img_no_scaled = ui.PreviewLabel->pixmap()->toImage();
}
	

QtCameraWidget::~QtCameraWidget()
{
}

void QtCameraWidget::SetCamera(const QCameraInfo& cameraInfo)
{

	m_camera.reset(new QCamera(cameraInfo));
	m_camera->setCaptureMode(QCamera::CaptureStillImage);

	m_imageCapture.reset(new QCameraImageCapture(m_camera.data()));

	if (m_imageCapture->isCaptureDestinationSupported(QCameraImageCapture::CaptureToBuffer))	
		m_imageCapture->setCaptureDestination(QCameraImageCapture::CaptureToBuffer);
	
	connect(m_imageCapture.data(), &QCameraImageCapture::imageCaptured, this, &QtCameraWidget::processCapturedImage);
	connect(m_imageCapture.data(), &QCameraImageCapture::imageSaved, this, &QtCameraWidget::imageSaved);

	m_camera->setViewfinder(ui.viewfinder);

	m_camera->start();

	imagecapture = m_imageCapture.data();
	
	ui.imageResolutionBox->clear();

	ui.imageResolutionBox->addItem(tr("Default Resolution"));
	const QList<QSize> supportedResolutions = imagecapture->supportedResolutions();
	for (const QSize &resolution : supportedResolutions) 
	{
		ui.imageResolutionBox->addItem(QString("%1x%2").arg(resolution.width()).arg(resolution.height()), QVariant(resolution));

	}

}

void QtCameraWidget::on_DoPhotoPushButton_clicked()
{
	m_imageCapture->capture();
}

void QtCameraWidget::on_SavePushButton_clicked()
{
	QByteArray bytes;
	QBuffer buffer(&bytes);
	buffer.open(QIODevice::WriteOnly);	
	ui.PreviewLabel->pixmap()->save(&buffer, "PNG");

	QSaveFile file(QFileDialog::getSaveFileName(NULL, "Save file", QDir::currentPath(), "PNG files (*.png)", new QString("PNG files (*.png)")));
	
	if (!file.fileName().isEmpty()) 
	{
	
		file.open(QIODevice::WriteOnly);

		file.write(bytes);
	
		file.commit();
	}

}

QByteArray QtCameraWidget::GetMyPhoto()
{
	QByteArray bytes;
	QBuffer buffer(&bytes);
	buffer.open(QIODevice::WriteOnly);
	//ui.PreviewLabel->pixmap()->save(&buffer, "PNG");
	img_no_scaled.save(&buffer, "PNG");
	return bytes;

}

void QtCameraWidget::CameraChange(int cameraIndex)
{
	QCameraInfo cameraInfo1;
	cameraInfo1 = qvariant_cast<QCameraInfo>(ui.CameraComboBox->currentData());
	SetCamera(cameraInfo1);
}

QVariant QtCameraWidget::boxValue(const QComboBox *box) const
{
	int idx = box->currentIndex();
	if (idx == -1)
		return QVariant();

	return box->itemData(idx);
}


void QtCameraWidget::DoResChange(int resindex)
{
	QImageEncoderSettings settings = imagecapture->encodingSettings();
	settings.setResolution(boxValue(ui.imageResolutionBox).toSize());

	m_imageCapture->setEncodingSettings(settings);
}


void QtCameraWidget::processCapturedImage(int requestId, const QImage& img)
{
	Q_UNUSED(requestId);

	QImage mirr;

	if (ui.MirrorCheckBox->isChecked())
		mirr = img.mirrored(true, false);
	else
		mirr = img.copy();
	
	
	img_no_scaled = mirr.copy();
	QImage scaledImage = mirr.scaled(ui.viewfinder->size(),Qt::KeepAspectRatio,Qt::SmoothTransformation);
	//QImage scaledImage = img;

	ui.PreviewLabel->setPixmap(QPixmap::fromImage(scaledImage));
}

void QtCameraWidget::imageSaved(int id, const QString& fileName)
{
	Q_UNUSED(id);
	
	
	QMessageBox msgBox;
	msgBox.setText(tr("Captured \"%1\"").arg(QDir::toNativeSeparators(fileName)));
	msgBox.exec();

	//m_isCapturingImage = false;
	//if (m_applicationExiting)
	//	close();
}

void QtCameraWidget::do_show()
{
	show();
}



