#include "QtChatWidget.h"

QtChatWidget::QtChatWidget(QWidget *parent)
	: QWidget(parent)
{
	ui.setupUi(this);
	load_skin(":/img/chat_fon.png");
	mymenu = new QMenu(this);

	ui.chatEdit->setStyleSheet("background: transparent");
	ui.ipEdit->setInputMask("000.000.000.000; _");
	ui.ipEdit->setCursorPosition(0);

	connect(ui.messageEdit, SIGNAL(returnPressed()), this, SLOT(returnPressed()));

	set_port(ui.portSpinBox->value());
}

QtChatWidget::~QtChatWidget()
{
	if (udpSocket)
	{
		udpSocket->deleteLater();
	}

	if (mymenu)
	{
		delete mymenu;
		mymenu = 0;
	}

}

void QtChatWidget::do_show()
{
	show();
}

void QtChatWidget::returnPressed()
{
	on_sendButton_clicked();
}

void QtChatWidget::readPendingDatagrams()
{
	while (udpSocket->hasPendingDatagrams())
	{
		QByteArray datagram;
		datagram.resize(udpSocket->pendingDatagramSize());
		QHostAddress* address = new QHostAddress();
		udpSocket->readDatagram(datagram.data(), datagram.size(), address);
		QDataStream in(&datagram, QIODevice::ReadOnly);
		
		QString now_time = QDateTime::currentDateTime().toString("dd.MM.yyyy HH:mm:ss");
		QString str_addres = QHostAddress(address->toIPv4Address()).toString();
		ui.chatEdit->append(str_addres+", "+now_time+ " '" + datagram.data()+"'");
	}
}


void QtChatWidget::set_port(const int& port)
{
	my_port = port;

	udpSocket = new QUdpSocket(this);

	if (!udpSocket->bind(QHostAddress::Any, my_port))
	{
		QTextCodec* codec = QTextCodec::codecForName("Windows-1251");
		QMessageBox::information(0, "Error", codec->toUnicode("   ") + QString::number(my_port));
	}

	connect(udpSocket, &QUdpSocket::readyRead, this, &QtChatWidget::readPendingDatagrams);
}

void QtChatWidget::change_port(const int& new_port)
{
		if (udpSocket)
			udpSocket->close();

		set_port(new_port);
}


void QtChatWidget::on_sendButton_clicked()
{
	if (!ui.messageEdit->text().isEmpty())
	{
		QByteArray data;
		data.append(ui.messageEdit->text());

		if (ui.for_ip->isChecked())
		{

			QHostAddress address(ui.ipEdit->text());
			udpSocket->writeDatagram(data, address, my_port);
			ui.chatEdit->append(ui.messageEdit->text());
		}
		else
			udpSocket->writeDatagram(data, QHostAddress::Broadcast, my_port);

		ui.messageEdit->clear();

	}
}

void QtChatWidget::on_ReconnectButton_clicked()
{
	change_port(ui.portSpinBox->value());
}

void QtChatWidget::on_chatEdit_customContextMenuRequested(QPoint)
{
	QTextCodec* codec = QTextCodec::codecForName("Windows-1251");
	mymenu->clear();
	mymenu->addAction(QIcon(tr(":/img/del_1.png")), codec->toUnicode(" "), this, SLOT(clear_all()));
	mymenu->exec(QCursor::pos());
}


void QtChatWidget::clear_all()
{
	ui.chatEdit->clear();
}

void QtChatWidget::paintEvent(QPaintEvent *)
{

	QPainter paint;
	paint.begin(this);
	paint.drawPixmap(QRect(0, 0, chat_bk.width(), chat_bk.height()), chat_bk);
	paint.end();
}

void QtChatWidget::resizeEvent(QResizeEvent *event)
{

	chat_bk = chat_bk_copy.scaled(width(), height(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
	QWidget::resizeEvent(event);

}

void QtChatWidget::load_skin(const QString &stName)
{
	chat_bk.load(stName);
	chat_bk_copy = chat_bk;
	chat_bk = chat_bk_copy.scaled(width(), height(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
}



