#include "QtWidgetsDemoClass.h"

#include <QApplication>
#include <QMainWindow>
#include <QPushButton>


QtWidgetsDemoClass::QtWidgetsDemoClass(QWidget* parent) : QWidget(parent)
{
	ui.setupUi(this);

	player = new QMediaPlayer;
	speech = new QTextToSpeech;

	QTextCodec* codec = QTextCodec::codecForName("Windows-1251");

	QSystemTrayIcon* trayIcon;
	QMenu* trayIconMenu;
	QAction* setupAction, * showCameraAction, * showChatAction;

	setupAction = new QAction(codec->toUnicode("&  "), this);
	setupAction->setIcon(QIcon(tr(":/img/show.png")));
	connect(setupAction, SIGNAL(triggered()), this, SLOT(do_show()));

	showCameraAction = new QAction(codec->toUnicode("&  "), this);
	showCameraAction->setIcon(QIcon(tr(":/img/camera.png")));
	connect(showCameraAction, SIGNAL(triggered()), this, SLOT(do_show_camera_widget()));

	showChatAction = new QAction(codec->toUnicode("&  "), this);
	showChatAction->setIcon(QIcon(tr(":/img/chat.png")));
	connect(showChatAction, SIGNAL(triggered()), this, SLOT(do_show_chat_widget()));

	//quitAction = new QAction(codec->toUnicode(""), this);
	//quitAction->setIcon(QIcon(tr(":/img/exit.png")));
	//connect(quitAction, &QAction::triggered, qApp, &QCoreApplication::quit);



	trayIconMenu = new QMenu(this);
	trayIconMenu->addAction(setupAction);
	trayIconMenu->addAction(showCameraAction);
	trayIconMenu->addAction(showChatAction);
	//trayIconMenu->addAction(quitAction);

	trayIcon = new QSystemTrayIcon(QIcon(":/img/qt.png"));
	trayIcon->setContextMenu(trayIconMenu);
	trayIcon->show();
}

QtWidgetsDemoClass::~QtWidgetsDemoClass()
{
	if (player)
	{
		delete player;
		player = 0;
	}
}

void QtWidgetsDemoClass::do_show()
{
	show();
}

void QtWidgetsDemoClass::do_show_camera_widget()
{
	emit show_camera_widget();
}


void QtWidgetsDemoClass::do_show_chat_widget()
{
	emit show_chat_widget();
}


void QtWidgetsDemoClass::on_pushButton_clicked()
{
	emit DoSendExternalEvent(ui.lineEditEvent->text(), ui.lineEditData->text());

}

void QtWidgetsDemoClass::on_sayButton_clicked()
{
	speech->say(ui.textEdit->toPlainText());
}


void QtWidgetsDemoClass::addTextEdittText(const QString my_data)
{
	ui.textEdit->append(my_data);
}

void QtWidgetsDemoClass::doPlaySound(const QString sndName)
{	
	player->stop();
	player->setMedia(QUrl::fromLocalFile(sndName));
	player->setVolume(100);
	player->play();	
}

void QtWidgetsDemoClass::doStopSound()
{
	player->stop();
}


